#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<errno.h>

#include<fcntl.h>
#include<unistd.h>
#include<signal.h>

#define DELAY		10
#define RESUMED_MARK	"PM: suspend exit"

#define perr(...) do { \
	fprintf(stderr, __VA_ARGS__);			\
	exit(-1);					\
} while (0)

unsigned long int gTestCount;
FILE *fp;

void
alarm_handler(int signo)
{
	(void)signo;
	perr("\n\nTIME OUT AFTER %lu tests\n\n", gTestCount);
	fprintf(fp, "\n\nTIME OUT AFTER %lu tests\n\n", gTestCount);
	fclose(fp);
	return;
}

void
get_line(int fd, char *buf, size_t len)
{
	for (size_t i = 0; i < len; i++) {
		ssize_t s = read(fd, buf + i, 1);
		if (s <= 0)
			perr("Cannot read file (%d)\n", errno);
		if (buf[i] == '\n') {
			buf[i] = '\0';
			return;
		}
	}
	perr("Line is too long\n");
}

int
main(int argc, const char *argv[])
{
	const char verbose_cmd[48] = "echo 9 9 9 9 > /proc/sys/kernel/printk\n";
	const char suspend_cmd[32] = "echo mem > /sys/power/state\n";
	const char fs_cmd[48] = "ls /mnt/; cat /mnt/hello.txt;\n";
	const char *wakeup_cmd = "\n\n\n\n";

	long time0 = 0;
	long time1 = 0;
	if (argc < 3)
		perr("Usage:\n\t%s <SERIAL_PORT> <STR_CHECK> <LOG_FILE>\n", argv[0]);

	int serial = open(argv[1], O_RDWR);
	if (serial < 0)
		perr("Cannot open serial port %s (%d)\n", argv[1], errno);

	if (argc > 3)
		fp = fopen(argv[3], "a+");
	else
		fp = fopen("log.txt", "a+");

	signal(SIGALRM, alarm_handler);

	write(serial, verbose_cmd, strlen(verbose_cmd) +1);
	write(serial, suspend_cmd, strlen(suspend_cmd) +1);

	while (1) {

		write(serial, suspend_cmd, strlen(suspend_cmd) +1);

		time0 = (random() % 13) + 2;
		sleep(time0);
		write(serial, wakeup_cmd, strlen(wakeup_cmd));

		sleep(1);

		alarm(20);
		char linebuf[4096];
		while (1) {
			get_line(serial, linebuf, 4096);
			puts(linebuf);
			fputs(linebuf, fp);
			fputc('\n', fp);
			fflush(fp);
			if (strstr(linebuf, argv[2]))
				break;
		}
		write(serial, fs_cmd, strlen(fs_cmd) +1);
		write(serial, wakeup_cmd, strlen(wakeup_cmd));
		alarm(0);
		time1 = (random() % 10) + 2;
		fprintf(fp, ">> info(%ld): sleep %ld, wake %ld\n", gTestCount, time0, time1);
		fflush(fp);
		sleep(time1);

		gTestCount++;
		printf("\n\n========== Test %lu Pass ================\n\n",
		       gTestCount);
	}

	return 0;
}
